//***********************************************************
//
//  NXV4 "Math-on-Paper" software
//
//  package : VEC class _____________________________________
//            
//            supports: move semantix
//                      operators + - * L,R-val context
//                                += -= *= /=
//                      fabs()
//                      ~  -      unary operators L,R-val
//                      mtx<arithmetic> i.e.     int
//                                               float 
//                                               double
//                                               real
//                                           cpx<int> 
//                                           cpx<float> 
//                                           cpx<double> 
//                                           cpx<real> 
//                      type_casts between the types 
//                      cout << overload 
//
//                      TOTAL = 2836 instantiated op/f's
//                      _____________________________________
//
//
//  authors : Mihai-Tiberiu Dima (1)
//            Maria Dima         (2)
//
//             Meshcheryakov Laboratory of Information Techn.
//             Joint Institute for Nuclear Research
//             Dubna-Russia
//
//            on leave from:
//
//             Hyperion Univ. Bucharest-Romania
//
//
//  date    : VEC4/v1.0.alfa   / Mon Mar 07 12:45:54 CET 2022
//
//***********************************************************
#pragma once


#include<iostream>
#include<cmath>
#include<type_traits>
#include<utility>

#include "cpx.hh"

using namespace cpx4                                        ;

using std::cout                                             ;
using std::endl                                             ;
using std::decay                                            ;
using std::declval                                          ;
using std::enable_if                                        ;
using std::is_same                                          ;
using std::forward                                          ;
using std::ostream                                          ;
using std::integral_constant                                ;
using std::remove_cv                                        ;

namespace mtx4
{
 template<typename T>
 class mtx                                                  ;
}

using namespace mtx4                                        ;

namespace vec4
{

 bool  utf8x = false                                        ; 

 using fltx  =      float                                   ;
 using dblx  =      double                                  ;
 using real  = long double                                  ;


template< class T >
struct is_cscalar: integral_constant<bool, 
 is_same<    int         , typename remove_cv<T>::type>::value ||
 is_same<    fltx        , typename remove_cv<T>::type>::value ||
 is_same<    dblx        , typename remove_cv<T>::type>::value ||
 is_same<    real        , typename remove_cv<T>::type>::value ||
 is_same<cpx<int >       , typename remove_cv<T>::type>::value ||
 is_same<cpx<fltx>       , typename remove_cv<T>::type>::value ||
 is_same<cpx<dblx>       , typename remove_cv<T>::type>::value ||
 is_same<cpx<real>       , typename remove_cv<T>::type>::value  >
{                                                           ;};

 template<typename scalar=dblx>
 class vec
   {public: typename decay<scalar>::type x = 0              ;
            typename decay<scalar>::type y = 0              ;
            typename decay<scalar>::type z = 0              ;
                               bool rangle = true           ;
      // 
      vec()                         // plain     
       {                                                    ;} 
      //
      vec(scalar const& a)
       {x = a                                               ;
        y = a                                               ;
        z = a                                               ;} 
      //
      template<typename U>
      vec(U const& a)
       {x = scalar(const_cast<U&>(  a))                     ;
        y = scalar(const_cast<U&>(  a))                     ;
        z = scalar(const_cast<U&>(  a))                     ;}
      //
      vec(scalar const& a, scalar const& aa, scalar const& aaa)
       {x =   a                                             ;
        y =  aa                                             ;
        z = aaa                                             ;}
      //
      template<typename U>
      vec(U const& a, U const& aa, U const& aaa)
       {x = scalar(const_cast<U&>(  a))                     ;
        y = scalar(const_cast<U&>( aa))                     ;
        z = scalar(const_cast<U&>(aaa))                     ;}
      //
      vec(const vec& A)             // copy       
       {x = A.x                                             ;
        y = A.y                                             ;
        z = A.z                                             ;
        rangle = A.rangle                                   ;}

      template<typename U>
      vec(vec<U> const& A)
       {x = scalar(const_cast<U&>(A.x))                     ;
        y = scalar(const_cast<U&>(A.y))                     ;
        z = scalar(const_cast<U&>(A.z))                     ;
        rangle = A.rangle                                   ;}

      // CAST  _____________________________________________

      explicit
      operator vec<int >()
      {return  vec<int >(int (x), int (y), int(z))          ;}
      explicit
      operator vec<fltx>()
      {return  vec<fltx>(fltx(x), fltx(y), fltx(z))         ;}
      explicit
      operator vec<dblx>()
      {return  vec<dblx>(dblx(x), dblx(y), dblx(z))         ;}
      explicit
      operator vec<real>()
      {return  vec<real>(real(x), real(y), real(z))         ;}

      explicit
      operator vec<cpx<int >>()
      {return  vec<cpx<int >>(cpx<int >(x), cpx<int >(y),
                                            cpx<int >(z))   ;}
      explicit
      operator vec<cpx<fltx>>()
      {return  vec<cpx<fltx>>(cpx<fltx>(x), cpx<fltx>(y),
                                            cpx<fltx>(z))   ;}
      explicit
      operator vec<cpx<dblx>>()
      {return  vec<cpx<dblx>>(cpx<dblx>(x), cpx<dblx>(y),
                                            cpx<dblx>(z))   ;}
      explicit
      operator vec<cpx<real>>()
      {return  vec<cpx<real>>(cpx<real>(x), cpx<real>(y),
                                            cpx<real>(z))   ;}

      // ASSGN _____________________________________________

      scalar& operator= (const scalar& A)
       {x = A                                               ;
        y = A                                               ;
        z = A                                               ;
        return const_cast<scalar&>(A)                       ;}

         vec& operator= (const    vec& A)
       {x = A.x                                             ;
        y = A.y                                             ;
        z = A.z                                             ;
        rangle = A.rangle                                   ;
        return const_cast<vec&>(A)                          ;}

      template<typename U> 
      vec<U>& operator= (vec<U> const& A)
       {x = scalar(const_cast<U&>(A.x))                     ;
        y = scalar(const_cast<U&>(A.y))                     ;
        z = scalar(const_cast<U&>(A.z))                     ;
        rangle = A.rangle                                   ;
        return const_cast<vec<U>&>(A)                       ;}

      // NGTV ______________________________________________

      template<typename T>
      friend
      vec<T>
      operator- (vec<T> const&)                             ;

      template<typename T>
      friend
      vec<T>&&
      operator- (vec<T> &&)                                 ;

      // TILDE (vec.conj.) _________________________________

      template<typename T>
      friend
      vec<T>
      operator~ (vec<T> const&)                             ;

      template<typename T>
      friend
      vec<T>&&
      operator~ (vec<T> &&)                                 ;

      // FABS ______________________________________________

      template<typename T>
      friend
      long double fabs(vec<T> const&)                       ;

      template<typename T>
      friend
      long double n2(vec<T> const&)                         ;

      // PROD ______________________________________________

      template<typename L, typename R>
      friend
      typename decay<decltype(declval<L>()*
                              declval<R>())>::type
      operator|(vec<L> const&, vec<R> const&)               ;

      // OPERATORS __________________________________________

         #include "eqal.cxx"
         #include "comp.cxx"

         #include "plus.cxx"
         #include "mnus.cxx"
         #include "mult.cxx"
         #include "divd.cxx"

      // ____________________________________________________

      friend
      std::ostream& operator<<(ostream&, vec<auto> const&)  ;

      // ____________________________________________________

      template<typename ALX, typename BLX>
      friend
      mtx<typename decay<decltype(declval<ALX>()*
                                  declval<BLX>() )
                        >::type
         >
      operator^(vec<ALX> const&, vec<BLX> const&)           ;

      // ____________________________________________________

     ~vec()
       {                                                    ;}
 };

//===========================================================
};
